﻿using System;
using MvpStoreFront.Business.ComponentInterfaces;
using MvpStoreFront.Business.Enumerations;
using MvpStoreFront.Business.ViewInterfaces;

namespace MvpStoreFront.Business.Presenters
{
    /// <summary>
    /// Marker interface for presenter class
    /// </summary>
    public interface IPresenter{}

    /// <summary>
    /// The base class for all presenters
    /// </summary>
    public abstract class BasePresenter<TView>: IPresenter where TView : class, IView
    {
        private TView _view;
        protected TView View
        {
            set
            {
                if (value == null)
                    throw new ArgumentException("View cannot be set to null");
                _view = value;
            }
            get
            {
                return _view;
            }

        }

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="BasePresenter&lt;TView&gt;"/> class.
        /// </summary>
        /// <param name="view">The view.</param>
        protected BasePresenter(TView view)
        {
            View = view;
                View.Load += OnViewLoad;
                SubscribeViewToEvents();
        }

        #endregion

        #region Shared External Dependencies


        private INorthwindRepository _NorthwindRepository;
        protected INorthwindRepository NorthwindRepository
        {
            get
            {
                if (_NorthwindRepository == null)
                    _NorthwindRepository = ServiceLocator.Instance().Resolve<INorthwindRepository>();
                return _NorthwindRepository;
            }
        }
	

        private INavigationComponent _NavigationComponent;
        protected INavigationComponent NavigationComponent
        {
            get
            {
                if (_NavigationComponent == null)
                    _NavigationComponent = ServiceLocator.Instance().Resolve<INavigationComponent>();
                return _NavigationComponent;
            }
        }


        #endregion

        #region Abstract Methods

        /// <summary>
        /// Subscribes the view to events.
        /// This method has to be implemented in every presenter class
        /// </summary>
        protected abstract void SubscribeViewToEvents();

        /// <summary>
        /// Called when the view is loaded.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected abstract void OnViewLoad(object sender, EventArgs e);

        /// <summary>
        /// Method that should be called to bind the model to the view
        /// </summary>
        public abstract void LoadViewFromModel();

        #endregion



        /// <summary>
        /// Indicating whether the view was originated due to post back.
        /// </summary>
        /// <value>
        /// 	<c>true</c> if this instance is postback; otherwise, <c>false</c>.
        /// </value>
        protected bool IsPostBack
        {
            get { return View.IsPostBack; }
        }

        /// <summary>
        /// Gets a parameter from the querystring.
        /// </summary>
        /// <param name="parameter">The named querystring parameter.</param>
        /// <returns></returns>
        protected string GetQueryParameter(QueryStringParameter parameter)
        {
            return View.QueryString[parameter.ToString()];    
        }

        /// <summary>
        /// Gets a parameter from the querystring.
        /// </summary>
        /// <param name="parameter">The named querystring parameter as an integer. 0 if no value is found</param>
        /// <returns></returns>
        protected int GetQueryParameterAsInt(QueryStringParameter parameter)
        {
            int result;
            int.TryParse(View.QueryString[parameter.ToString()], out result);
            return result;
        }



    }
}