﻿

using System.Collections.Generic;
using System.Collections.Specialized;
using System.Text;
using System.Web;
using System.Linq;
using System.Web.UI;
using System.Web.UI.WebControls;
using MvpStoreFront.Business.ComponentInterfaces;
using MvpStoreFront.Business.Enumerations;
using QueryStringParameter = MvpStoreFront.Business.Enumerations.QueryStringParameter;

namespace MvpStoreFront.Web.Code
{
    /// <summary>
    /// Component used for navigating in the webapp
    /// </summary>
    internal class NavigationComponent : INavigationComponent
    {
        /// <summary>
        /// The dictionary containing all phiysical paths to the pages in the webapp
        /// Could be private, but made internal so the unit tests can access it to verify that ALL page identifiers are used and the aspx's exist
        /// Unused page identifiers should be removed, as well as unused pages - so we keep the app free from dead and unused links
        /// </summary>
        internal static readonly IDictionary<PageIdentifier, string> pages =
            new Dictionary<PageIdentifier, string>(){
                    {PageIdentifier.Home, "~/Home.aspx"},
                    {PageIdentifier.Categories, "~/Categories.aspx"},
                    {PageIdentifier.List, "~/List.aspx"},
                    {PageIdentifier.Detail, "~/Detail.aspx"}
                                                    };

        /// <summary>
        /// The dictionary containing all phiysical paths to the usercontrols in the webapp
        /// Could be private, but made internal so the unit tests can access it to verify that ALL control identifiers are used and the ascx's exist
        /// Unused control identifiers should be removed, as well as unused controls - so we keep the app free from dead and unused links
        /// </summary>
        internal static readonly IDictionary<ControlIdentifier, string> controls =
            new Dictionary<ControlIdentifier, string>()
            {
            };


        /// <summary>
        /// Initializes a new instance of the <see cref="NavigationComponent"/> class.
        /// </summary>
        public NavigationComponent()
        {

        }

        /// <summary>
        /// Refreshes the current page (redirect, no postback !).
        /// </summary>
        public void Refresh()
        {
            HttpContext.Current.Response.Redirect(HttpContext.Current.Request.Url.PathAndQuery, true);
        }

        private const string ReturnUrl = "ReturnUrl";

        /// <summary>
        /// Navigates to a specified page, stocking the current Url in the querystring, so we can return later, calling "ReturnToCaller".
        /// </summary>
        /// <param name="pageIdentifier">The page identifier.</param>
        public void NavigateAndReturnTo(PageIdentifier pageIdentifier)
        {
            NavigateToCore(pageIdentifier, ReturnUrl, HttpUtility.UrlEncode(HttpContext.Current.Request.RawUrl));
        }

        /// <summary>
        /// Gets a previously stocked Url from the querystring, and navigates to it.
        /// </summary>
        public bool ReturnToCaller()
        {
            if (HttpContext.Current.Request.QueryString.AllKeys.Contains(ReturnUrl))
            {
                HttpContext.Current.Response.Redirect((HttpContext.Current.Request.QueryString[ReturnUrl]));
                return true;
            }
            return false;
        }

        /// <summary>
        /// Navigates to a page uniquely identified by the page identifier, passing the specified querystring parameters as a namevaluecollection.
        /// </summary>
        /// <param name="pageIdentifier">The page identifier.</param>
        /// <param name="queryString">The query string.</param>
        public void NavigateTo(PageIdentifier pageIdentifier, NameValueCollection queryString)
        {
            NavigateToCore(pageIdentifier, GetParams(queryString));
        }

        /// <summary>
        /// Navigates to a page uniquely identified by the page identifier, passing the specifief querystring parameters.
        /// </summary>
        /// <param name="pageIdentifier">The page identifier.</param>
        public void NavigateTo(PageIdentifier pageIdentifier)
        {
            NavigateToCore(pageIdentifier);
        }

        /// <summary>
        /// Navigates to a page uniquely identified by the page identifier, passing the specifief querystring parameters.
        /// </summary>
        /// <param name="pageIdentifier">The page identifier.</param>
        /// <param name="param1">The param1.</param>
        /// <param name="value1">The value1.</param>
        public void NavigateTo(PageIdentifier pageIdentifier, QueryStringParameter param1, string value1)
        {
            NavigateToCore(pageIdentifier, param1.ToString(), value1);
        }

        /// <summary>
        /// Navigates to a page uniquely identified by the page identifier, passing the specifief querystring parameters.
        /// </summary>
        /// <param name="pageIdentifier">The page identifier.</param>
        /// <param name="param1">The param1.</param>
        /// <param name="value1">The value1.</param>
        /// <param name="param2">The param2.</param>
        /// <param name="value2"></param>
        public void NavigateTo(PageIdentifier pageIdentifier, QueryStringParameter param1, string value1, QueryStringParameter param2, string value2)
        {
            NavigateToCore(pageIdentifier, param1.ToString(), value1, param2.ToString(), value2);
        }

        /// <summary>
        /// Navigates to a page uniquely identified by the page identifier, passing the specifief querystring parameters.
        /// </summary>
        /// <param name="pageIdentifier">The page identifier.</param>
        /// <param name="param1">The param1.</param>
        /// <param name="value1">The value1.</param>
        /// <param name="param2">The param2.</param>
        /// <param name="value2">The value2.</param>
        /// <param name="param3">The param3.</param>
        /// <param name="value3">The value3.</param>
        public void NavigateTo(PageIdentifier pageIdentifier, QueryStringParameter param1, string value1, QueryStringParameter param2, string value2, QueryStringParameter param3, string value3)
        {
            NavigateToCore(pageIdentifier, param1.ToString(), value1, param2.ToString(), value2, param3.ToString(), value3);
        }


        /// <summary>
        /// Navigates to a page uniquely identified by the page identifier, passing the specified querystring as a string.
        /// </summary>
        /// <param name="pageIdentifier">The page identifier.</param>
        /// <param name="queryParams">The query string parameters, even params = identifiers, odd params = values.</param>
        private void NavigateToCore(PageIdentifier pageIdentifier, params string[] queryParams)
        {
            HttpContext.Current.Response.Redirect(GetUrlCore(pageIdentifier, queryParams), true);
        }
        /// <summary>
        /// Gets the URL for the specified page.
        /// </summary>
        /// <param name="identifier">The page identifier.</param>
        /// <param name="queryString">The query string.</param>
        /// <returns></returns>
        public string GetUrl(PageIdentifier identifier, NameValueCollection queryString)
        {
            return GetUrlCore(identifier, GetParams(queryString));
        }

        /// <summary>
        /// Stronlgy type GetUrl overloads
        /// </summary>
        /// <param name="identifier">The identifier.</param>
        /// <returns></returns>
        public string GetUrl(PageIdentifier identifier)
        {
            return GetUrlCore(identifier);
        }
        /// <summary>
        /// Stronlgy typed GetUrl overloads
        /// </summary>
        /// <param name="identifier">The identifier.</param>
        /// <param name="param1">The param1.</param>
        /// <param name="value1">The value1.</param>
        /// <returns></returns>
        public string GetUrl(PageIdentifier identifier, QueryStringParameter param1, string value1)
        {
            return GetUrlCore(identifier, param1.ToString(), value1);
        }
        /// <summary>
        /// Stronlgy typed GetUrl overloads
        /// </summary>
        /// <param name="identifier">The identifier.</param>
        /// <param name="param1">The param1.</param>
        /// <param name="value1">The value1.</param>
        /// <param name="param2">The param2.</param>
        /// <param name="value2">The value2.</param>
        /// <returns></returns>
        public string GetUrl(PageIdentifier identifier, QueryStringParameter param1, string value1, QueryStringParameter param2, string value2)
        {
            return GetUrlCore(identifier, param1.ToString(), value1, param2.ToString(), value2);
        }
        /// <summary>
        /// Stronlgy typed GetUrl overloads
        /// </summary>
        /// <param name="identifier">The identifier.</param>
        /// <param name="param1">The param1.</param>
        /// <param name="value1">The value1.</param>
        /// <param name="param2">The param2.</param>
        /// <param name="value2">The value2.</param>
        /// <param name="param3">The param3.</param>
        /// <param name="value3">The value3.</param>
        /// <returns></returns>
        public string GetUrl(PageIdentifier identifier, QueryStringParameter param1, string value1, QueryStringParameter param2, string value2, QueryStringParameter param3, string value3)
        {
            return GetUrlCore(identifier, param1.ToString(), value1, param2.ToString(), value2, param3.ToString(), value3);
        }


        /// <summary>
        /// Gets the URL for the specified page.
        /// </summary>
        /// <param name="identifier">The page identifier.</param>
        /// <param name="queryParams">The query string parameters, even params = identifiers, odd params = values.</param>
        /// <returns></returns>
        private string GetUrlCore(PageIdentifier identifier, params string[] queryParams)
        {
            return (pages.ContainsKey(identifier) ? BuildUrl(pages[identifier], queryParams) : null);
        }

        /// <summary>
        /// Gets the URL for the specified control.
        /// </summary>
        /// <param name="identifier">The control identifier.</param>
        /// <returns></returns>
        public string GetUrl(ControlIdentifier identifier)
        {
            return (controls.ContainsKey(identifier) ? controls[identifier] : null);
        }

        /// <summary>
        /// internal helperfunction to convert a namevaluecollection of queryString params to a sting array of these params
        /// </summary>
        /// <param name="queryString">the string array of queryParams</param>
        /// <returns></returns>
        private string[] GetParams(NameValueCollection queryString)
        {
            string[] queryParams = new string[queryString.Count * 2];
            for (int i = 0; i < queryString.Count; i++)
            {
                queryParams[i * 2] = queryString.GetKey(i);
                queryParams[(i * 2) + 1] = queryString[i];
            }
            return queryParams;
        }
        /// <summary>
        /// internal helper function to actually assemble the url with the query parameters
        /// </summary>
        /// <param name="url">The URL.</param>
        /// <param name="queryParams">The query params.</param>
        /// <returns></returns>
        private string BuildUrl(string url, params string[] queryParams)
        {
            StringBuilder sb = new StringBuilder(url);
            if (queryParams.Length > 0)
            {
                sb.Append("?");
                bool alternatingChar = true;
                foreach (string param in queryParams)
                {
                    sb.Append(param);
                    sb.Append(alternatingChar ? "=" : "&");
                    alternatingChar = !alternatingChar;
                }
                //remove the last character (= or &)
                sb.Remove(sb.Length - 1, 1);
            }
            Control c = new Control();
            return c.ResolveClientUrl(sb.ToString());
        }
    }
}